/*
 * Author:
 *   Andy Li
 *   andy@onthewings.net
 *   http://www.onthewings.net/
 */

import Box2D.Collision.Shapes.b2CircleDef;
import Box2D.Collision.Shapes.b2PolygonDef;
import Box2D.Collision.b2AABB;
import Box2D.Common.Math.b2Vec2;
import Box2D.Dynamics.b2Body;
import Box2D.Dynamics.b2BodyDef;
import Box2D.Dynamics.b2World;

import flash.display.Sprite;

import org.casalib.util.NumberUtil;
import org.casalib.util.StageReference;

[Bindable]
public var numOfAtom:uint = 4000;
[Bindable]
public var atomRadius:Number = 3;

public var m_world:b2World;

public function init():void {
	StageReference.setStage(systemManager.stage);

	var worldAABB:b2AABB = new b2AABB();
	worldAABB.lowerBound.Set(-100.0, -100.0);
	worldAABB.upperBound.Set(100.0, 100.0);

	var gravity:b2Vec2 = new b2Vec2(0.0, 0.0);

	m_world = new b2World(worldAABB, gravity, true);
}

public function enterFrame(evt:Event = null):void {
	m_world.Step(1 / StageReference.getStage().frameRate, 2, 8);

	// Go through body list and update sprite positions/rotations
	var sprite:Sprite;
	var numSleep:uint = 0;
	for (var bb:b2Body = m_world.GetBodyList(); bb; bb = bb.GetNext()) {
		if (bb.GetUserData() is Sprite) {
			sprite = bb.GetUserData();
			if (bb.IsSleeping()){
				++numSleep;
				sprite.visible = true;
				sprite.x = bb.GetPosition().x * 30;
				sprite.y = bb.GetPosition().y * 30;
				sprite.rotation = bb.GetAngle() * (180 / Math.PI);
			} else {
				sprite.visible = false;
			}
		}
	}
	contactCountLabel.text = "contact count: " + m_world.GetContactCount().toString();
	sleepCountLabel.text = "sleep count: " + numSleep.toString();
}

public function startSim():void {
	var bodyDef:b2BodyDef = new b2BodyDef();
	var boxDef:b2PolygonDef = new b2PolygonDef();
	var b2W:Number, b2H:Number;
	var flashSprite:Sprite;
	var body:b2Body;

	//add some balls
	var ballDef:b2CircleDef = new b2CircleDef();
	ballDef.radius = atomRadius / 30;
	ballDef.friction = 0.1;
	ballDef.density = 1;
	for (var ballCount:Number = 0; ballCount < numOfAtom; ++ballCount) {
		bodyDef.position.Set(NumberUtil.randomWithinRange(StageReference.getStage().stageWidth / 30 * 0.2, StageReference.getStage().stageWidth / 30 * 0.8), NumberUtil.randomWithinRange(StageReference.getStage().stageHeight / 30 * 0.2, StageReference.getStage().stageHeight / 30 * 0.8));
		//bodyDef.isBullet = true;
		
		flashSprite = new Sprite();
		flashSprite.graphics.beginFill(0);
		//flashSprite.graphics.drawRect(ballDef.radius*30*-0.5, ballDef.radius*30*-0.5, ballDef.radius*30*2, ballDef.radius*30*2);
		flashSprite.graphics.drawCircle(0, 0, atomRadius);
		bodyDef.userData = flashSprite;
		rawChildren.addChild(flashSprite);

		body = m_world.CreateBody(bodyDef);
		body.CreateShape(ballDef);
		body.SetMassFromShapes();
	}
	
	panel.visible = false;
	
	enterFrame();
	addEventListener(Event.ENTER_FRAME, enterFrame);
}
